@@ -1,5 +1,7 @@
 /***************************************************************************
-* Copyright (c) 2016, Wolf Vollprecht, Johan Mabille and Sylvain Corlay    *
+* Copyright (c) Johan Mabille, Sylvain Corlay, Wolf Vollprecht and         *
+* Martin Renou                                                             *
+* Copyright (c) QuantStack                                                 *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
@@ -36,6 +38,7 @@ namespace xsimd
         using self_type = batch<int64_t, 2>;
         using base_type = simd_batch<self_type>;
         using storage_type = typename base_type::storage_type;
+        using batch_bool_type = typename base_type::batch_bool_type;
 
         batch();
         explicit batch(int64_t src);
@@ -50,6 +53,9 @@ namespace xsimd
         batch(const storage_type& rhs);
         batch& operator=(const storage_type& rhs);
 
+        batch(const batch_bool_type& rhs);
+        batch& operator=(const batch_bool_type& rhs);
+
         operator storage_type() const;
 
         XSIMD_DECLARE_LOAD_STORE_ALL(int64_t, 2)
@@ -110,6 +116,25 @@ namespace xsimd
         return *this;
     }
 
+    namespace detail
+    {
+        inline int64x2_t init_from_bool(uint64x2_t a)
+        {
+            return vandq_s64(reinterpret_cast<int64x2_t>(a), vdupq_n_s64(1));
+        }
+    }
+
+    inline batch<int64_t, 2>::batch(const batch_bool_type& rhs)
+        : base_type(detail::init_from_bool(rhs))
+    {
+    }
+
+    inline batch<int64_t, 2>& batch<int64_t, 2>::operator=(const batch_bool_type& rhs)
+    {
+        this->m_value = detail::init_from_bool(rhs);
+        return *this;
+    }
+
     XSIMD_DEFINE_LOAD_STORE(int64_t, 2, int8_t, XSIMD_DEFAULT_ALIGNMENT)
     XSIMD_DEFINE_LOAD_STORE(int64_t, 2, uint8_t, XSIMD_DEFAULT_ALIGNMENT)
     XSIMD_DEFINE_LOAD_STORE(int64_t, 2, int16_t, XSIMD_DEFAULT_ALIGNMENT)
